use core_graphics::display::{
    kCGNullWindowID, kCGWindowImageDefault, kCGWindowListOptionOnScreenOnly, CGDisplay,
};
use paris::info;
use scrap::{Capturer, Display};
use tracing::debug;

use super::{config::DisplayConfig, screen::Screen, screenshot::Screenshot};

pub struct DisplayPicker {
    pub screen: Screen,
    pub config: DisplayConfig,
}

impl DisplayPicker {
    pub fn from_config(config: DisplayConfig) -> anyhow::Result<Self> {
        let displays = Display::all()
            .map_err(|error| anyhow::anyhow!("Can not get all of displays. {}", error))?;
        let display = displays.into_iter().skip(config.index_of_display).next();

        match display {
            Some(display) => {
                let height = display.height();
                let width = display.width();
                info!("dw: {}, cw: {}", width, config.display_height);
                assert_eq!(width, config.display_width);
                let capturer = Capturer::new(display)?;
                let screen = Screen::new(capturer, width, height);

                Ok(Self { screen, config })
            }
            None => {
                anyhow::bail!("Index out of displays range.")
            }
        }
    }

    pub fn take_screenshot(&mut self) -> anyhow::Result<Screenshot> {
        debug!("take_screenshot");
        let start_at = std::time::Instant::now();

        let cg_display = CGDisplay::new(self.config.id);
        let cg_image = CGDisplay::screenshot(
            cg_display.bounds(),
            kCGWindowListOptionOnScreenOnly,
            kCGNullWindowID,
            kCGWindowImageDefault,
        )
        .ok_or_else(|| anyhow::anyhow!("Display#{}: take screenshot failed", self.config.id))?;
        debug!("take screenshot took {}ms", start_at.elapsed().as_millis());

        let buffer = cg_image.data();
        let bytes_per_row = cg_image.bytes_per_row() as f32;

        let height = cg_image.height();
        let width = cg_image.width();

        let scale_factor = self.config.scale_factor;
        let image_height = (height as f32 / scale_factor) as u32;
        let image_width = (width as f32 / scale_factor) as u32;

        debug!(
            "raw image: {}x{}, output image: {}x{}",
            width, height, image_width, image_height
        );
        // // from bitmap vec
        let mut image_buffer = vec![0u8; (image_width * image_height * 3) as usize];

        for y in 0..image_height {
            for x in 0..image_width {
                let offset =
                    (((y as f32) * bytes_per_row + (x as f32) * 4.0) * scale_factor) as usize;
                let b = buffer[offset];
                let g = buffer[offset + 1];
                let r = buffer[offset + 2];
                let a = buffer[offset + 3];
                let offset = (y * image_width + x) as usize;
                image_buffer[offset * 3] = r;
                image_buffer[offset * 3 + 1] = g;
                image_buffer[offset * 3 + 2] = b;
            }
        }
        debug!(
            "convert to image buffer took {}ms",
            start_at.elapsed().as_millis()
        );

        // println!("encode to png took {}ms", start_at.elapsed().as_millis());
        // // // base64 image
        // let mut image_base64 = String::new();
        // image_base64.push_str("data:image/png;base64,");
        // let encoded = base64::engine::general_purpose::STANDARD_NO_PAD.encode(image_png);
        // image_base64.push_str(encoded.as_str());

        // println!("took {}ms", start_at.elapsed().as_millis());
        // println!("image_base64: {}", image_base64.len());

        // info!("bitmap size {}", bitmap.len());
        let screenshot = Screenshot::new(image_buffer, self.config);
        Ok(screenshot)
    }
}
